#' @export
QPdecon <-
function(Y, Pdf=TRUE, K=NULL, f_Z, lambda="SURE", reg="2Deriv", integr=TRUE, nonneg=TRUE, monotone=NULL, unimod=FALSE, convex=NULL, support=NULL){
	
	N=length(Y)
	if(is.null(K)) K = min(200, floor(3 * sqrt(N)))

	#Call find.CDd() to compute the following information, to be used for computing various quantities in the QP optimization
	CDd <- find.CDd(Y=Y, Pdf=Pdf, K=K, f_Z=f_Z, reg=reg, support=support) 
	x <- CDd$x  #K-length vector of x values over which f_X is estimated
	x0 <- CDd$x0 #K_sup-length vector of x values for only the specified support of X. All elements of x0 are a subset of x
	delta <- CDd$delta #x discretization interval
	f_Y <- CDd$f_Y #Estimated density of Y, from its histogram
	C <- CDd$C #KxK_sup Convolution matrix C
	K_sup <- CDd$K_sup #number of discrete x values over the support
	K <- CDd$K #number of discrete x values over the entire range
	l_sup <- CDd$l_sup #lower boundary of support
	u_sup <- CDd$u_sup #upper boundary of support
	CtC <- CDd$CtC  #t(C)%*%C
	CtfY <- CDd$CtfY #t(C)%*%f_Y
	CtSig <- CDd$CtSig #t(C)%*%Sigma, where Sigma is the covariance matrix of f_Y
	DtD <- CDd$DtD  #the K_supxK_sup matrix in the quadratic regularization (I, t(D2)%*%D2, etc) 
	f_reg <- CDd$f_reg #density to which f_X is regularized. Zero-vector if reg == "2Deriv"

	#Call find.Ab() to compute the following constraint information, to be used in the QP optimization
	Ab <- find.Ab(Pdf=Pdf, K_sup=K_sup, integr=integr, nonneg=nonneg, monotone=monotone, convex=convex, l_sup=l_sup, u_sup=u_sup, x0=x0)
	meq <- Ab$meq   ##the number of equality constraints (1 or 0, the former if integr==TRUE
	A <- Ab$A  ##A consraint matrix to be used in solve.QP
	b <- Ab$b  ##b constraint vector to be used in solve.QP

	#If lambda = "SURE", find the SURE estimate of lambda. This ignores the unimod constraint, if present.
	if(lambda=="SURE"){
		opt.fun <- function(mu, ...) {out = f_XandSURE(mu, ...); out$SURE.value} ##create wrapper function for f_XandSURE for optimization
		OPT=optimize(opt.fun, c(-20,5), f_Y=f_Y, Pdf=Pdf, integr=integr, D=NULL, d=NULL, C=C, CtC=CtC, DtD=DtD, CtfY=CtfY, CtSig=CtSig, f_reg=f_reg, A=A, b=b, meq=meq)
		lambda=exp(OPT$minimum) ##The SURE estimate of lambda
	} else if (lambda<0) {
		stop("lambda must be nonnegative")
	}

	#For the single value of lambda (either the SURE value or a single specified value), compute the D matrix and d vector used in the objective function when calling solve.QP. 
	D <- CtC + lambda*DtD
	d <- CtfY + lambda*f_reg

	#If unimod == TRUE, optimize the Unimodal() function to find mymode, set the value for monotone based on this, and recompute the A and b contraint information.
	#First do some error checking and find the lowerbound and upper bound for mymode in the optimization search.
	if((unimod == TRUE) && (Pdf == TRUE)){ ##This block sets lower/upper search bounds and conducts the Unimodal optimization

		if(!is.null(monotone) || !is.null(convex)){ ##If either monotone or convex constraints are specified
			l_mon <- as.numeric(monotone[1])			
			u_mon <- monotone[2]
			l_con<- as.numeric(convex[1])			 
			u_con <- convex[2]
			if(is.null(convex) && !is.null(monotone)){ ##If monotone constraint is specified and convex is not
			
		     		if(!is.na(u_mon) && !is.na(l_mon)){
					if((l_mon<l_sup) || (u_mon>u_sup)) stop("monotone constraint invalid, exceeds lower or upper bound!")
					lowerbound=l_mon
					upperbound=u_mon
				}else if(!is.na(l_mon) && is.na(u_mon)){
					if(l_mon<l_sup) stop("monotone constraint invalid, exceeds lower bound!")
					lowerbound=l_mon
					upperbound=u_sup
				}else if(!is.na(u_mon) && is.na(l_mon)){
					if(u_mon>u_sup) stop("monotone constraint invalid, exceeds upper bound!")
					lowerbound=l_sup
					upperbound=u_mon
				}else{
					stop("monotone constraint invalid, both tails are missing!")
				}

			}else if(is.null(monotone) && !is.null(convex)){ ##If convex constraint is specified and monotone is not
			
		     		if(!is.na(u_con) && !is.na(l_con)){
					if((l_con<l_sup) || (u_con>u_sup)) stop("convex constraint invalid, exceeds lower or upper bound!")
					lowerbound=l_con; upperbound=u_con
				}else if(!is.na(l_con) && is.na(u_con)){
					if(l_con<l_sup) stop("convex constraint invalid, exceeds lower bound!")
					lowerbound=l_con
					upperbound=u_sup
				}else if(!is.na(u_con) && is.na(l_con)){
					if(u_con>u_sup) stop("convex constraint invalid, exceeds upper bound!")
					lowerbound=l_sup
					upperbound=u_con 
				}else{
					stop("convex constraint invalid, both tails are missing!")
				}
				
			}else{ ##If both monotone and convex constraints are specified

				if((is.na(u_mon) && is.na(l_mon)) || (is.na(u_con)&&is.na(l_con))) stop("convex or monotone constraint invalid, both tails are missing!")		
				if(!is.na(l_mon) && !is.na(u_mon)){
					if(u_mon>u_sup || l_mon<l_sup) stop("monotone constraint invalid, exceeds lower or upper bound!")
					if(!is.na(l_con) && !is.na(u_con)){
						if(u_con>u_sup || l_con<l_sup) stop("convex or monotne constraint invalid, exceeds lower or upper bound!")
						lowerbound=max(l_mon,l_con)
						upperbound=min(u_mon,u_con)
					}else{	
						if(is.na(l_con)){
					      	if(u_con>u_sup) stop("convex constraint invalid, exceeds upper bound!")					
							lowerbound=l_mon
							upperbound=min(u_con,u_mon)
						}else{ ##If u_con == NA
							if(l_con<l_sup) stop("convex constraint invalid, exceeds lower bound!")
							lowerbound=max(l_mon,l_con)
							upperbound=u_mon
						}
					} 
				}else if(is.na(u_mon) && !is.na(l_mon)){
					if(l_mon<l_sup) stop("monotne constraint invalid, exceeds lower bound!")
					if(!is.na(l_con) && !is.na(u_con)){
						if(u_con>u_sup || l_con<l_sup) stop("convex or monotne constraint invalid, exceeds lower or upper bound!")
						lowerbound=max(l_mon,l_con)
						upperbound=u_con
					}else{	
						if(is.na(l_con)) {
					      	if(u_con>u_sup) stop("convex constraint invalid, exceeds upper bound!")					
							lowerbound=l_mon
							upperbound=u_con
						}else{##is.na(u_con)
							if(l_con<l_sup) stop("convex constraint invalid, exceeds lower bound!")	
							lowerbound=max(l_mon,l_con)
							upperbound=u_sup
						}
					} 
				}else if(!is.na(u_mon) && is.na(l_mon)){
					if(u_mon>u_sup) stop("monotne constraint invalid, exceeds upper bound!")
					if(!is.na(l_con) && !is.na(u_con)){
						if(u_con>u_sup || l_con<l_sup) stop("convex or monotne constraint invalid, exceeds lower or upper bound!")
						lowerbound=l_con
						upperbound=min(u_mon, u_con)
					}else{	
						if(is.na(l_con)){
					      	if(u_con>u_sup) stop("convex constraint invalid, exceeds upper bound!")					
							lowerbound=l_sup
							upperbound=min(u_mon,u_con)
						}else{##is.na(u_con)
							if(l_con<l_sup) stop("convex constraint invalid, exceeds lower bound!")	
							lowerbound=l_con
							upperbound=u_mon
						}
					} 
				}
			} ##End of block specifying lower/upper search bounds if either monotone or convex constraints are used

		}else{ ##Set lower/upper search bounds if neither monotone or convex constraints are specified
			lowerbound <- l_sup
			upperbound <- u_sup
		} ##End of block setting lower and upper search bounds for the Unimodal optimization

		##Conduct the Unimodal optimization using the lower/upper search bounds just found and reset monotone argument appropriately
		opti<-optimize(Unimodal,c(lowerbound,upperbound), D=D, d=d, Pdf=Pdf, K_sup=K_sup, integr=integr, nonneg=nonneg, convex=convex, l_sup=l_sup, u_sup=u_sup, x0=x0)
		mymode=opti$minimum
		monotone=c(mymode,mymode)

		##Call find.Ab() to recompute the constraint information used in the QP optimization with the new monotone argument from the unimodal optimization
		Ab <- find.Ab(Pdf=Pdf, K_sup=K_sup, integr=integr, nonneg=nonneg, monotone=monotone, convex=convex, l_sup=l_sup, u_sup=u_sup, x0=x0)
		meq <- Ab$meq   ##the number of equality constraints (1 or 0, the former if integr==TRUE
		A <- Ab$A  ##A consraint matrix to be used in solve.QP
		b <- Ab$b  ##b constraint vector to be used in solve.QP

	}else if((unimod == TRUE) && (Pdf == FALSE)){
		stop("Unimod == TRUE constraint is not used when Pdf == FALSE")
	}else { ##If unimod == FALSE, set mymode = "N/A"
		mymode="N/A"
	}# end of unimod block

	##For the single value of lambda (either SURE or single specified value) and the updated A and b constraint information, estimate f_X and its SURE value 
	OUT <- f_XandSURE(mu=log(lambda), f_Y=f_Y, Pdf=Pdf, integr=integr, D=D, d=d, C=C, CtC=CtC, DtD=DtD, CtfY=CtfY, CtSig=CtSig, f_reg=f_reg, A=A, b=b, meq=meq)
	SURE.value <- OUT$SURE.value #value of SURE criterion
	f_X <- OUT$f_X ##estimated K_sup-length pdf f_X over only the support x0

	##Convert f_X from a K_sup-length vector over x0 to a K-length vector over the entire range x
	if (Pdf==TRUE){
		f_X=c(rep(0,length(x[x<x0[1]])),f_X,rep(0,length(x[x>x0[K_sup]])))
	}else{
		f_X=c(rep(0,length(x[x<x0[1]])),f_X,rep(1,length(x[x>x0[K_sup]])))
	}

	QP=NULL
	QP$x=x
	QP$f_X=f_X
	QP$SURE.value=SURE.value
	QP$f_Y=f_Y
	QP$lambda=lambda
	QP$mymode=mymode

	class(QP) = "QPdecon"

	return(QP)

}
