\name{QPdecon}
\alias{QPdecon}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Density deconvolution using quadratic programming
}
\description{
%QPdecon is a function that solves a deconvolution problem with regularization to estimate the density of X in the model Y = X + Z with noise Z, given a sample of observations of Y. The optimization problem is formulated as \eqn{\|\hat{f}_Y-Cf_X\|^2+\lambda Q(f_X)}, and the function finds a \eqn{\hat{f}_X} to minimize above quadratic programming problem subject to various constraints
QPdecon is a function that solves a deconvolution problem to estimate the density of X in the model Y = X + Z with noise Z, given a sample of observations of Y. The deconvolution problem is formulated as a quadratic programming problem with regularization, and the function finds a discretized version of f_X to minimize the quadratic programming problem subject to various constraints

}
\usage{
QPdecon(Y, Pdf = TRUE, K = NULL, f_Z, lambda = "SURE", reg = "2Deriv", integr = TRUE, nonneg = TRUE, monotone = NULL, unimod = FALSE, convex = NULL, support = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Y}{
	N-length vector of N observations of Y
}
  \item{Pdf}{
	set \code{Pdf=TRUE} to obtain the pdf estimator, or set \code{Pdf=FALSE} to obtain the cdf estimator   
}
  \item{K}{
	number of discrete points at which the pdfs will be evaluated over the range of X and Y

}
  \item{f_Z}{
	the pdf of the noise Z. For zero-mean Gaussian Z, \code{f_Z} is a constant that represents the standard deviation of Z. For any other noise distribution, \code{f_Z} should be a user-defined function that takes a scalar (say \eqn{z}) as an input argument and returns a scalar value for the noise density f_Z(\eqn{z}) at \eqn{Z = z}

}
  \item{lambda}{
	To use a specific value for the regularization parameter, set \code{lambda} equal to the value. Otherwise, to have \code{lambda} chosen automatically (by the SURE method), set \code{lambda = "SURE"}

}
  \item{reg}{
	regularization method can be set as \code{reg = "Gauss", "2Deriv"} or \code{"Unif"}. \code{reg="Gauss"} sets the regularization to be |f_X-f_reg|^2, where f_reg is a Gaussian distribution having the same mean and variance as X (which is determined internally, based on the sample variance of Y and the noise variance from \code{f_Z}); \code{reg = "2Deriv"}, which is the default, penalizes the magnitude of the second-derivative of \code{f_X};  \code{reg="Unif"} sets f_reg as a uniform distribution over the range of Y
}
  \item{integr}{
	if \code{Pdf=TRUE}, setting \code{integr=TRUE} enforces the integrate-to-one constraint on \code{f_X}; if \code{Pdf=FALSE}, i.e. to obtain cdf estimator, then setting \code{integr=TRUE} enforces the last element in the vector \code{f_X} to be 1
}
  \item{nonneg}{
	set \code{nonneg=TRUE} to include the nonnegativity constraint that each element of \code{f_X} is nonnegative
}
  \item{monotone}{
	set as a two-length vector \code{monotone=c(l_mon,u_mon)} to specify the monotonic regions. \code{f_X} is constrained to be monotonically increasing for \code{x} below \code{l_mon} and decreasing above \code{u_mon}. If \code{monotone=NULL}, no monotonicity constraints will be used. For only upper-tail monotonicity, specify \code{l_mon} as \code{NA}, and let \code{u_mon} equal the value after which the pdf is monotonically decreasing. Likewise, for lower-tail monotonicity, let \code{l_mon} equal the value below which the pdf is monotonically increassing, and set \code{u_mon} as \code{NA}  
}
  \item{unimod}{
	set to \code{TRUE} to enforce the pdf estimator is unimodal; not active for cdf estimator
}
  \item{convex}{
	set as a two-length vector \code{convex=c(l_con,u_con)} to specify the convex regions. \code{f_X} is constrained to be convex for \code{x} below \code{l_con} and above \code{u_con}. If \code{convex=NULL}, no convexity constraints will be used. For only upper-tail convexity, specify \code{l_con} as \code{NA}, and let \code{u_con} equal the value after which the pdf is convex. Likewise, for lower-tail convexitity, let \code{l_con} equal the value below which the pdf is convex, and set \code{u_con} as \code{NA}
}
  \item{support}{
	set as a two-length vector \code{support=c(l_sup,u_sup)} to specify the lower and upper end of the support; the default is \code{support=c(min(Y),max(Y))}; the lower and upper end can be specified to be \code{-Inf} or \code{Inf}, respectively  
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
	The function returns an object of class \code{QPdecon}, which is composed of the following attributes:
	\item{x }{input domain: \code{K}-lenth vector of evenly-spaced values in the observed range of Y}
	\item{f_X }{\code{K}-length vector of the estimated pdf or cdf of X, at values in \code{x}}
	\item{SURE.value}{value of SURE criterion}
	\item{f_Y }{\code{K}-length vector of the  estimated pdf or cdf of Y from its histogram, evaluated at the values in \code{x}}
	\item{lambda}{automatically selected or user-specified value of regularization parameter}
	\item{mymode}{the automatically estimated mode location when \code{unimod} constraint is \code{TRUE}; otherwise, \code{mymode="N/A"}}

}
\references{
	Ran Yang, Daniel Apley, Jeremy Staum and David Ruppert. Density Deconvolution with Additive Measurement Errors using Quadratic Programming.

}
\author{
	Ran Yang, D. W. Apley
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{plot.QPdecon}} for plotting the QP estimator
}
\examples{
rm(list=ls())

X=rgamma(3000,5,1)
Z=rnorm(3000,0,sd=1.8)
Y=X+Z
f_Z<-function(z){dnorm(z,mean=0,sd=1.8)}
K=min(200,floor(3*sqrt(3000)))

#### QP estimation ####
QPobj=QPdecon(Y=Y,Pdf=TRUE,K=K,f_Z=f_Z,lambda=0.02,reg="2Deriv",integr=TRUE,
nonneg=TRUE)

#### Plot the estimation ####
plot(QPobj,histo=TRUE)
#### compare with the true pdf ####
lines(QPobj$x,dgamma(QPobj$x,5,1),lwd=1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }% use one of  RShowDoc("KEYWORDS")
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
