
# Investigates robustness to misspecification of f_Z
# f_X = normal mixture
# Two-letter codes: true f_z/assumed f_z
# G = Gaussian, L = Laplace

library(QPdecon)
library(deamer)
library(mixtools)
cdf2quant = function(xx,cdf,pr)
{
  q = matrix(nrow=length(pr),ncol=1)
  for (i in 1:length(pr))
  {
    ind = (cdf>pr[i])
    q[i] = min(xx[ind])
  }
  q
}
set.seed(38538)
qq = c(0.025,0.05,0.1,0.25,0.5,0.75,0.9,0.95,0.975)

###  Define model
n = 1000
sdz = 0.75
sdx = sqrt(0.8*0.2*4^2 + 1)
grid = seq(-1,3,length = 500)*sdx
TRUEcdf = 0.8*pnorm(grid,mean=1/sdx,sd=1/sdx) + 0.2*pnorm(grid,mean=5/sdx,sd=1/sdx)
TRUEquant = cdf2quant(grid,TRUEcdf,qq)
rmodel = function(n) {
  rnormmix(n, lambda=c(0.8,0.2), mu=c(1,5)/sdx, sigma=1/sdx)
}
dmodel = function(x) 0.8*dnorm(x,mean=1/sdx,sd=1/sdx) + 0.2*dnorm(x,mean=5/sdx,sd=1/sdx)
truePDF = dmodel(TRUEquant) 
###  End define model

Nsim = 200
K = 200
QPpdf11 = matrix(nrow=Nsim,ncol=length(TRUEquant))
QPpdf12 = matrix(nrow=Nsim,ncol=length(TRUEquant))
QPpdf21 = matrix(nrow=Nsim,ncol=length(TRUEquant))
QPpdf22 = matrix(nrow=Nsim,ncol=length(TRUEquant))
for (isim in 1:Nsim)
{
  x = rmodel(n)
  z1 = rnorm(n,mean=0,sd=sdz)
  c = sdz/sqrt(2)
  z2 = rlaplace(n,b=c) 
  f_Z1 = sdz
  f_Z2 <- function(z) dlaplace(z,mu=0,b=c)
  y1 = x + z1
  y2 = x + z2
  fitQP11 = QPdecon(y1, Pdf = TRUE, K = K, f_Z=f_Z1, 
                  lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                  nonneg = TRUE,
                  support = range(y1))
  fitQP12 = QPdecon(y1, Pdf = TRUE, K = K, f_Z=f_Z2, 
                    lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                    nonneg = TRUE,
                    support = range(y1))
  fitQP21 = QPdecon(y2, Pdf = TRUE, K = K, f_Z=f_Z1, 
                    lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                    nonneg = TRUE,
                    support = range(y2))
  fitQP22 = QPdecon(y2, Pdf = TRUE, K = K, f_Z=f_Z2, 
                    lambda = "SURE", reg = "2Deriv", integr = TRUE, 
                    nonneg = TRUE,
                    support = range(y2))
  QPcdf11 = cumsum(fitQP11$f_X); QPcdf11 = QPcdf11/QPcdf11[length(QPcdf11)]
  QPpdf11[isim,] = spline(y=fitQP11$f_X,x=fitQP11$x,xout=TRUEquant)$y
  
  QPcdf12 = cumsum(fitQP12$f_X); QPcdf12 = QPcdf12/QPcdf12[length(QPcdf12)]
  QPpdf12[isim,] = spline(y=fitQP12$f_X,x=fitQP12$x,xout=TRUEquant)$y
  
  QPcdf21 = cumsum(fitQP21$f_X); QPcdf21 = QPcdf21/QPcdf21[length(QPcdf21)]
  QPpdf21[isim,] = spline(y=fitQP21$f_X,x=fitQP21$x,xout=TRUEquant)$y
  
  QPcdf22 = cumsum(fitQP22$f_X); QPcdf22 = QPcdf22/QPcdf22[length(QPcdf22)]
  QPpdf22[isim,] = spline(y=fitQP22$f_X,x=fitQP22$x,xout=TRUEquant)$y
  print(isim)
}

par(mfrow=c(2,2))
plot(TRUEquant,QPpdf11[2,])
plot(TRUEquant,QPpdf12[2,])
plot(TRUEquant,QPpdf21[2,])
plot(TRUEquant,QPpdf22[2,])

MAE11 = abs(QPpdf11 - matrix(t(truePDF),nrow=Nsim,ncol=length(qq),byrow = TRUE))
MAE11 = median(rowMeans(MAE11))

MAE12 = abs(QPpdf12 - matrix(t(truePDF),nrow=Nsim,ncol=length(qq),byrow = TRUE))
MAE12 = median(rowMeans(MAE12))

MAE21 = abs(QPpdf21 - matrix(t(truePDF),nrow=Nsim,ncol=length(qq),byrow = TRUE))
MAE21 = median(rowMeans(MAE21))

MAE22 = abs(QPpdf22 - matrix(t(truePDF),nrow=Nsim,ncol=length(qq),byrow = TRUE))
MAE22 = median(rowMeans(MAE22))

MAEs = c(MAE11,MAE12,MAE21,MAE22)

names(MAEs)=names = c("GG","GL","LG","LL")
round(MAEs,3)

save("MAEs",file="robustNormMix.RData")
